/* ------------------------------------------------------------ */
/*
HTTrack, Offline Browser for Windows and Unix
Copyright (C) Xavier Roche & Yann Philippot

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


Important notes:

- We hereby ask people using this source NOT to use it in purpose of grabbing
emails addresses, or collecting any other private informations on persons.
This would disgrace our work, and spoil the many hours we spent on it.


This project has been developed by Xavier Roche and Yann Philippot,
and originally supervised by Daniel Carr and Patrick Ducrot.
Developed at "ISMRA - Ensi Caen", computing department
(Engineering School of Caen - France, http://www.ismra.fr)
*/


/* ------------------------------------------------------------ */
/* File: httrack.c subroutines:                                 */
/*       filters ("regexp")                                     */
/* ------------------------------------------------------------ */


#include "htsfilters.h"

//  partir d'un tableau de {"+*.toto","-*.zip","+*.tata"} dfinit si nom est autoris
int fa_strjoker(char** filters,int nfil,char* nom) {
  int verdict = 0;  // on sait pas
  int i;
  for(i=0;i<nfil;i++) {
    if (strjoker(nom,filters[i] + 1)) {    // reconnu
      if (filters[i][0]=='+')
        verdict = 1;    // autoris
      else
        verdict = -1;   // interdit
    }
  }
  return verdict;
}


// supercomparateur joker (tm)
// compare a et b (b=avec joker dedans), case insensitive [voir CI]
// renvoi l'adresse de la premire lettre de la chaine
// (cd *[..]toto.. renvoi adresse de toto dans la chaine)
// accepte les dlires du genre www.*.*/ * / * truc*.*
// cet algo est 'un peu' rcursif mais ne consomme pas trop de tm
// * = toute lettre
// --?-- : spcifique  HTTrack et aux ?
HTS_INLINE char* strjoker(char* chaine,char* joker) {
  int err=0;
  if (strlen(joker)==0) {         // fin de chaine joker
    if (strlen(chaine)==0)        // fin aussi pour la chaine: ok
      return chaine;
    else if (chaine[0]=='?')
      return chaine;  // --?-- pour les index.html?Choix=2
    else
      return NULL;    // non trouv
  }

  // on va progresser en suivant les 'mots' contenus dans le joker
  // un mot peut tre un * ou bien toute autre squence de lettres
  
  if (strcmp(joker,"*")==0) {  // ok, rien aprs
    return chaine;
  }
  
  // 1er cas: jokers * ou jokers multiples *[..]
  if (joker[0]=='*') {  // comparer joker+reste (*toto/..)
    int jmp;    // nombre de caractres pour le prochain mot dans joker
    int cut = 0;  // interdire tout caractre superflu
    char pass[256];
    if ((joker[1]=='[') && (joker[2]!='[')) {  // multijoker (tm)
      int i;
      for(i=0;i<256;i++) pass[i]=0;

      // noms rservs
      if ((strfield(joker+2,"file")) || (strfield(joker+2,"name"))) {
        for(i=0;i<256;i++) pass[i]=1;
        pass[(int) '?'] = 0;
        pass[(int) ';'] = 0;
        pass[(int) '/'] = 0;
        i=2;
        while ((joker[i]!=']') && (joker[i]) && (i<(int) strlen(joker))) i++;
      } else if (strfield(joker+2,"path")) {
        for(i=0;i<256;i++) pass[i]=1;
        pass[(int) '?'] = 0;
        pass[(int) ';'] = 0;
        i=2;
        while ((joker[i]!=']') && (joker[i]) && (i<(int) strlen(joker))) i++;
      } else if (strfield(joker+2,"param")) {
        if (chaine[0]=='?') {  // il y a un paramtre juste l
          for(i=0;i<256;i++) pass[i]=1;
        }  // sinon synonyme de 'rien'
        i=2;
        while ((joker[i]!=']') && (joker[i]) && (i<(int) strlen(joker))) i++;
      } else {
        // dcode les directives comme *[A-Z,,0-9]
        i=2;
        if (joker[i] == ']') {    // *[] signifie "plus rien aprs"
          cut = 1;    // caractre supplmentaire interdit
        } else {
          while ((joker[i]!=']') && (joker[i]) && (i<(int) strlen(joker))) {
            if (joker[i+1]=='-') {  // 2 car, ex: *[A-Z]
              if ((int) (unsigned char) joker[i+2]>(int) (unsigned char) joker[i]) {
                int j;
                for(j=(int) (unsigned char) joker[i];j<=(int) (unsigned char) joker[i+2];j++)
                  pass[j]=1;
                
              } else err=1;
              i+=3;
            } else {            // 1 car, ex: *[ ]
              pass[(int) (unsigned char) joker[i]]=1;
              i++;
            }
            if ((joker[i]==',') || (joker[i]==';')) i++;
          }
        }
      }
      //  sauter dans joker
      jmp=i;
      if (joker[i]) jmp++; 
      
      //
    } else {  // tout autoriser
      //
      int i;
      for(i=0;i<256;i++) pass[i]=1;    // tout autoriser
      jmp=1;
      if (joker[2]=='[') jmp=3;        // permet de recher *<crochet ouvrant>
    }
    
    {
      int i,max;
      char* adr;

      // la chaine doit se terminer exactement
      if (cut) {
        if (strlen(chaine)>0)
          return NULL;    // perdu
        else
          return chaine;  // ok
      }

      // comparaison en boucle, c'est ca qui consomme huhu..
      // le tableau pass[256] indique les caractres ASCII autoriss

      // tester sans le joker (pas ()+ mais ()*)
      if (adr=strjoker(chaine,joker+jmp)) {
        return adr;
      }
      // tester
      i=0; max=strlen(chaine);
      while(i<(int) max) {
        if (pass[(int) (unsigned char) chaine[i]]) {  // caractre autoris
          if (adr=strjoker(chaine+i+1,joker+jmp)) {
            return adr;
          }
          i++;
        } else i=max+2;  // sortir
      }
      // tester chane vide
      if (i!=max+2)  // avant c'est ok
      if (adr=strjoker(chaine+max,joker+jmp))
        return adr;

      return NULL;  // perdu
    }

  } else {  // comparer mot+reste (toto*..)
    if (strlen(chaine)>0) {
      int jmp=0,ok=1;
      
      // comparer dbut de joker et dbut de chaine
      while((joker[jmp]!='*') && (joker[jmp]) && (ok)) {
        // CI : remplacer streql par une comparaison !=
        if (!streql(chaine[jmp],joker[jmp])) {
          ok=0;  // quitter
        }
        jmp++;
      }
      
      // comparaison ok?
      if (ok) {
        // continuer la comparaison.
        if (strjoker(chaine+jmp,joker+jmp))
          return chaine;    // retourner 1e lettre
      }
      
    }  // strlen(a)
    return NULL;   
  }  // * ou mot

  return NULL;
}

// recherche multiple
// exemple: find dans un texte de strcpy(*[A-Z,a-z],"*[0-9]"); va rechercher la premire occurence
// d'un strcpy sur une variable ayant un nom en lettres et copiant une chaine de chiffres
// ATTENTION!! Eviter les jokers en dbut, o gare au temps machine!
char* strjokerfind(char* chaine,char* joker) {
  char* adr;
  while(*chaine) {
    if (adr=strjoker(chaine,joker)) {  // ok trouv
      return adr;
    }
    chaine++;
  }
  return NULL;
}
